@@ -1297,7 +1297,6 @@ gst_tag_list_add_valist (GstTagList * list, GstTagMergeMode mode,
       g_warning ("unknown tag '%s'", tag);
       return;
     }
-    memset (&value, 0, sizeof (value));
     G_VALUE_COLLECT_INIT (&value, info->type, var_args, 0, &error);
     if (error) {
       g_warning ("%s: %s", G_STRLOC, error);
@@ -2186,3 +2185,9 @@ gst_tag_list_take (GstTagList ** old_taglist, GstTagList * new_taglist)
   return gst_mini_object_take ((GstMiniObject **) old_taglist,
       (GstMiniObject *) new_taglist);
 }
+
+const GstStructure *
+_gst_tag_list_structure (const GstTagList * list)
+{
+  return GST_TAG_LIST_STRUCTURE (list);
+}
